import json
import random

def wait_for_user_input(name: str):
    while True:
        user_input = input(f"[?] {name}? (Y/N): ")
        if user_input.lower() == "y":
            print(f"[i] {name} ENABLED")
            return True
        elif user_input.lower() == "n":
            print(f"[i] {name} disabled")
            return False

def wait_for_user_start() -> bool:
    if automatic:
        return True
    else:
        return wait_for_user_input("Start")

def get_do_disable_first() -> bool:
    # This is probably unneeded...
    # I forgot why I put this in V1 so uhhh I'm too scared to take it out now
    if automatic:
        return False
    return wait_for_user_input("Disable all peers first")

def get_do_randomize() -> bool:
    if automatic:
        return do_randomize;
    return wait_for_user_input("Randomize ports")

def get_peer_selection(length: int) -> int:
    if automatic:
        return random.randint(0, length - 1)
    else:
        user_in = int(input("Select the target server [index] (-1 for random): "))
        if user_in == -1:
            return random.randint(0, length - 1)
        else:
            return user_in - 1  # Index starts at 1

def get_do_speedtest() -> bool:
    if automatic:
        return True # I guess this is what you are here for anyways
    else:
        return wait_for_user_input("Use speedtest to automatically select server")


def get_speedtest_threshold() -> float:
    """
    :returns in mbps.
    """
    return speedtest_threshold * 1000.0 * 1000.0

def get_gateway_name() -> str:
    return gateway_name


def genconfig(location:str) -> None:
    data = {
        "keepalive_interval": 25, # The keepalive_interval of your peers. There's no good way of retrieving this info from OPNSense last time I checked.
        "randomize_port_range": [[53, 53], [123, 123], [443, 443], [4000, 33433], [33565, 51820], [52001, 60000]], # Preset ranges for Mullvad. Inclusive.
        "instance_UUID": "[CHANGE ME]", # Find it via https://github.com/KipJM/OpnsenseMullvadPeerAdder/blob/master/config_and_run_me_first.py
        "opnsense_api": "[CHANGE ME, EXAMPLE: https://192.168.1.1/api]",
        "api_file": "api_key.txt",
        "debug": False,  # OPTIONAL. Shows debug info. Does not impact functionality.
        "automatic": False,  # OPTIONAL. Override all user input will preset values above.
        "speedtest_threshold": 25,  # in mbps. Once a peer reached at least this bandwidth the tool will stop searching any further
        "speedtest_maxtries": 5, # Max number of different peers to try before giving up.
        "gateway_name": "",  # OPTIONAL. Will attempt to reset the gateway watcher for this to remove the orange dot on the dashboard.
        "do_randomize": False,  # OPTIONAL. Randomize ports of all peers
        "verbose": False
    }
    with open(location, "w+") as f2:
        # load default values
        json.dump(data, f2)

    print(f"sample json generated at {location}. You need to modify some of its values.")


def load_config():
    global keepalive_interval, randomize_port_range, instance_UUID, opnsense_api, speedtest_threshold, speedtest_maxtries, debug, automatic, gateway_name, do_randomize, verbose, api_key, api_secret
    from . import cli_config

    with open(cli_config.config_path, "r") as f:
        data = json.load(f)
        if data == {}:
            print(
                "[ERR] CONFIG FILE DOES NOT EXIST/EMPTY. Generate a template config using: opnspeedguard --genconfig --config (where to output the file)")
            exit(-1)

    keepalive_interval = data["keepalive_interval"]  # Default: 25
    randomize_port_range = data[
        "randomize_port_range"]  # Mullvad allowed port ranges. You can remove any undesired port range here.
    instance_UUID = data["instance_UUID"]
    opnsense_api = data["opnsense_api"]
    speedtest_threshold = data["speedtest_threshold"]  # MB
    speedtest_maxtries = data["speedtest_maxtries"]

    # OPTIONALS
    debug = data.get("debug", False)
    automatic = data.get("automatic", False)
    gateway_name = data.get("gateway_name", "")
    do_randomize = data.get("do_randomize", False)
    verbose = data.get("verbose", False)

    # Read api key
    try:
        with open(cli_config.api_path) as f:
            api_key = f.readline().strip().removeprefix("key=")
            api_secret = f.readline().strip().removeprefix("secret=")
    except FileNotFoundError:
        print("[ERR] API KEY FILE NOT FOUND! Assign a valid api_key.txt generated by OPNSense!")
        exit(-1)


# == MAIN ===

keepalive_interval      = None
randomize_port_range    = None
instance_UUID           = ""
opnsense_api            = ""
speedtest_threshold     = None
speedtest_maxtries      = None
debug                   = False
automatic               = False
gateway_name            = ""
do_randomize            = False
verbose                 = False
api_key                 = ""
api_secret              = ""


from . import config

print ("Creating speedtest client")
import speedtest
sp_client = speedtest.Speedtest(secure=True)
print("Getting speedtest servers...")
sp_client.get_servers()

def speedtest() -> float:
    print("Getting best server...")
    sp_client.get_best_server()
    if config.debug:
        print(f"Closest server: {sp_client.closest}")

    print("Starting download speedtest.")
    dl_speed = sp_client.download()

    return dl_speed

def format_bps(bps: float) -> str:
    if bps > 1000 * 1000 * 1000:
        return f"{"%.1f" % (bps/1000/1000/1000)}gbps"
    elif bps > 1000 * 1000:
        return f"{"%.1f" % (bps/1000/1000)}mbps"
    elif bps > 1000:
        return f"{"%.1f" % (bps/1000)}kbps"
    else:
        return f"{"%.1f" % bps}bits/s"


"""
OPNSpeedGuard
By KIP

:3
"""

import argparse
import os
import pathlib

def run():
    from . import cli_config
    parser = argparse.ArgumentParser(prog="opnspeedguard",
                                     description="Automatically switches an OPNSense's WireGuard connection to a fast peer, for use in road warrior setups. (c) KIP")
    parser.add_argument("--genconfig", action="store_true",
                        help="Generate a template config.json file to be used with -config")
    parser.add_argument("--config", type=str,
                        help="Path to REQUIRED config json file that controls auto connect behavior. If not specified, the program will try to use config.json in the working directory.")
    parser.add_argument("--api", type=str,
                        help="Path to REQUIRED api_key.txt file generated by OPNSense. If not specified, the program will try to use api_key.txt in the working directory.")

    args = parser.parse_args()

    print("== PRE-RUN INFO ==")
    # Gen config
    if args.genconfig:
        if args.config is not None:
            genconfig_location = args.config
            # GEN CONFIG
            print(f"Generating template config file to {genconfig_location}...")
        else:
            genconfig_location = "config.json"
            print(f"Generating template config file to {os.getcwd()}...")

        from .config import genconfig
        genconfig(genconfig_location)
        exit(2)

    # config
    if args.config is None:
        print("--config is not specified. Fallback to config.json in working directory.")
        if not os.path.exists("config.json"):
            print(
                f"[ERR] Cannot find fallback config.json. Either set --config, or place config.json at {os.getcwd()}.")
            print(
                "You can generate a template config file using opnspeedguard --genconfig --config=output_file_location")
            exit(-1)
        else:
            cli_config.config_path = "config.json"
    else:
        if not os.path.exists(args.config):
            print(f"[ERR] Cannot find config file located at {args.config}. Check your arguments.")
            exit(-1)
        else:
            cli_config.config_path = args.config

    # api key
    if args.api is None:
        print("--api is not specified. Fallback to api_key.txt in working directory.")
        if not os.path.exists("api_key.txt"):
            print(f"[ERR] Cannot find fallback api_key.txt. Either set --api, or place api_key.txt at {os.getcwd()}")
            print(
                "This file can be retrieved from the OPNSense web panel, via System > Access > Users > Ticket icon (Create and Download API keys...).")
            exit(-1)
        else:
            cli_config.api_path = "api_key.txt"
    else:
        if not os.path.exists(args.api):
            print(f"[ERR] Cannot find api key file located at {args.api}. Check your arguments.")
            exit(-1)
        else:
            cli_config.api_path = args.api

    print("====")
    from . import auto_connect
    auto_connect.auto_connect()

if __name__ == "__main__":
    run()